package org.jdom;

import java.util.List;
import java.util.ListIterator;
import java.util.ArrayList;

public class NodeIterator
{
	protected NodeX current;
	
	protected List level;
	
	protected ListIterator liter;
	
	public NodeIterator(NodeX node)
	{
		current = node;
		adjust();
	}
	
	
    public NodeX goDown()
	{		
		if (current == null)
			throw new RuntimeException("Can't go down!");
			
		level = current.goDown();
		liter = level.listIterator();
		
		if (liter.hasNext())
		{
			current = (NodeX)liter.next();
			 
		    return current;
		}
		else
			throw new RuntimeException("Can't go down!");
	}
	
	
	public boolean hasDown()
	{
		if (current == null)
			return false;
			
		return current.goDown().size() > 0;
	}
	
	
	public NodeX goUp()
	{
		if (current == null)
			throw new RuntimeException("Can't go up!");
			
		current = current.goUp();
		adjust();
		
	    return current;	
	}
	
	
	public boolean hasUp()
	{
		if (current == null)
			return false;
			
		return current.goUp() != null;
	}
	
	
	public NodeX goForward()
	{
		if (current == null)
			throw new RuntimeException("Can't go forward!");
			
		if (!liter.hasNext())
			throw new RuntimeException("Can't go forward!");
			
		current = (NodeX)liter.next();
		
		return current;
	}
	
	
	public boolean hasForward()
	{
		if (current == null)
			return false;
			
		return liter.hasNext();
	}
	
	public NodeX goBack()
	{
		if (current == null)
			throw new RuntimeException("Can't go back!");
			
		if (!liter.hasPrevious())
			throw new RuntimeException("Can't go back!");
			
		current = (NodeX)liter.previous(); 
		
		return current;
	}
	
	
	public boolean hasBack()
	{
		if (current == null)
			return false;
			
		return liter.hasPrevious();
	}
	
	protected void adjust()
	{
		if (hasUp())
		{
			level = current.goUp().goDown();
			liter = level.listIterator();
			
			NodeX maybe = null;
			
			do
			{
				maybe = (NodeX)liter.next();
			}
			while (maybe != current);	
		}
		else
		{
		    level = new ArrayList();        // Never null
		    liter = level.listIterator();
		}
	}
}