package org.jdom;

import java.util.List;

/**
 * <code>NodeX</code> acts as the base interface for all the XML content types.
 * 
 * @author Todd Trimmer
 * @version 1.0
 */
public interface NodeX {
	
	/**
	 * <p>
	 * Go up to parent node.
	 * </p>
	 * 
	 * @return parent.
	 */
	public NodeX goUp();
	
	/**
	 * <p>
	 * Go down to first child node.
	 * </p>
	 * 
	 * @return first child.
	 */
	public List goDown();

	/**
	 * <p>
	 * Allow a visitor to peek at this node.
	 * </p>
	 * 
	 * @param visitor <code>NodeVisitor</code> wanting to peek at this node.
	 * @return Arbitrary <code>Object</code> returned from the visitor.
	 * @see org.jdom.NodeVisitor org.jdom.NodeVisitor
	 */
	public Object acceptVisitor(NodeVisitor visitor);
	
	/**
	 * <p>
	 * Return a <code>NodeIterator</code> that will traverse through all the
	 * children of this node.
	 * </p>
	 * 
	 * @return <code>NodeIterator</code> to traverse this node's children.
	 * @see org.jdom.NodeIterator org.jdom.NodeIterator
	 */
	public NodeIterator nodeIterator();
	
	/** Silly debugger method */
	public String debug();
}